"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const errors_1 = require("@oclif/errors");
const chalk_1 = require("chalk");
const cheerio = require("cheerio");
const fs = require("fs-extra");
const globby = require("globby");
const Listr = require("listr");
const path = require("path");
const base_1 = require("../base");
const utils_1 = require("../utils"); //tslint:disable-line no-unused no-unused-variable
class Reconfigure extends base_1.default {
    constructor() {
        super(...arguments);
        this.hasUrlMappingsConfig = async (root) => {
            const files = await globby('Custom/Config/urlMappings.config', { cwd: root, nocase: true });
            return files.length > 0;
        };
        this.createUrlMappingsConfig = async (root) => {
            const $ = cheerio.load('<?xml version="1.0"?>\r\n<urlMappings />', {
                xmlMode: true
            });
            const contents = $.xml();
            const filepath = path.join(root, 'Custom/Config/urlMappings.config');
            return new Listr([{
                    title: chalk_1.default `Create empty {bold urlMappings.config}`,
                    task: () => fs.writeFile(filepath, contents),
                }]).run();
        };
        this.getConfigurationFiles = async () => {
            const coreRepo = await this.getCorePath();
            const globtions = {
                expandDirectories: true,
                nocase: true,
            };
            const linkedRepoRoot = await this.getLinkedRepoRoot();
            return Promise.all([
                globby(`${coreRepo}/**/connectionStrings.config`, globtions),
                globby(`${coreRepo}/**/app.config`, globtions),
                globby(`${linkedRepoRoot}/**/app.config`, globtions),
                globby(`${coreRepo}/**/smtp.config`, globtions)
            ])
                .then(mdArray => {
                return mdArray.reduce((acc, arr) => {
                    return acc.concat(arr);
                }, []);
            });
        };
        this.processFile = async (filepath, ctx) => {
            let numReplacements = 0;
            const contents = await fs.readFile(filepath, 'utf8');
            const { server, database, maildrop, } = ctx;
            const { replace: replaceFlag, 'replace-raw': replaceRawFlag, } = this.flags;
            const stripBom = !this.flags['no-strip-bom'];
            const $ = cheerio.load(contents, {
                xmlMode: true
            });
            $('connectionStrings > [connectionString]').attr('connectionString', (_i, val) => {
                const retval = replaceFlag || replaceRawFlag || val;
                if (!retval.match(/uid=/i)) {
                    this.warn(chalk_1.default `{yellow Connection string does not contain a standard username assignment ("uid=")}: \n  {gray -> String:} {bold "${retval}"}\n  {gray -> in file:} {bold ${filepath}}`);
                    // this.warn(chalk`Connection string does not contain a username: "${retval}"\n\t↦ in file {bold ${filepath}}`)
                }
                if (!retval.match(/pwd=/i)) {
                    this.warn(chalk_1.default `{yellow Connection string does not contain a standard password assignment ("pwd=")}: \n  {gray -> String:} {bold "${retval}"}\n  {gray -> in file:} {bold ${filepath}}`);
                }
                if (replaceRawFlag) {
                    numReplacements += 1;
                    return retval;
                }
                return retval
                    .replace(/(Server|Data Source)=.+?;/gi, (_match, p1) => {
                    numReplacements += 1;
                    return `${p1}=${server};`;
                })
                    .replace(/(DataBase|Initial Catalog)=.+?;/gi, (_match, p1) => {
                    numReplacements += 1;
                    return `${p1}=${database};`;
                });
            });
            if (maildrop) {
                $('smtp specifiedPickupDirectory[pickupDirectoryLocation]').attr('pickupDirectoryLocation', maildrop);
            }
            let modifiedContents = $.xml();
            if (stripBom) {
                modifiedContents = modifiedContents.replace(/^&#xFEFF;/, '');
            }
            await fs.writeFile(filepath, modifiedContents);
            return numReplacements;
        };
    }
    async run() {
        await this.requireConfig();
        let server = this.flags.server;
        let database = this.flags.database;
        if (!server) {
            server = this.userConfig.server;
        }
        if (!database) {
            database = this.userConfig.database;
        }
        console.log('');
        if (!server) {
            throw new errors_1.CLIError('No database server configured. Try rerunning `rtolms init` to set one or pass --server');
        }
        if (!database) {
            throw new errors_1.CLIError('No database configured. Try rerunning `rtolms init` to set one or pass --dbName');
        }
        this.log(chalk_1.default `{gray Changing {reset server} => {green ${server}}}\n`);
        this.log(chalk_1.default `{gray Changing {reset database} => {green ${database}}}\n`);
        const linkedRepoRoot = await this.getLinkedRepoRoot();
        if (linkedRepoRoot) {
            const hasUrlMappings = await this.hasUrlMappingsConfig(linkedRepoRoot);
            if (!hasUrlMappings) {
                this.warn(chalk_1.default `{yellow The currently linked custom repo does not contain a {bold urlMappings.config}. It will be created.}`);
                this.log('');
                await this.createUrlMappingsConfig(linkedRepoRoot);
                this.log('');
            }
        }
        else {
            this.warn(chalk_1.default `{yellow WARNING:} There is no currently linked custom repo.`);
        }
        const filesToMutate = await this.getConfigurationFiles();
        if (filesToMutate.length === 0) {
            throw new errors_1.CLIError('No configuration files found to change with the currently linked customer.');
        }
        return new Listr(filesToMutate.map(filepath => {
            const realpath = path.relative(this.userConfig.reposDir, fs.realpathSync(filepath));
            return {
                title: utils_1.decorateRepoPath(realpath),
                task: async (ctx, task) => {
                    const numReplacements = await this.processFile(filepath, ctx);
                    const replacements = numReplacements > 0 ? chalk_1.default `{bold {green ${numReplacements.toString().padStart(2)}}}` : chalk_1.default `{gray --}`;
                    task.title = chalk_1.default `{gray [${replacements}]} ${utils_1.decorateRepoPath(realpath)}`;
                },
            };
        })).run({
            server,
            database,
            maildrop: this.userConfig.maildrop,
        })
            .then(() => {
            this.log('\nDone.');
        });
    }
}
Reconfigure.description = 'Changes database connection strings';
Reconfigure.aliases = ['reconf', 'reconfig', 'setdb', 'set-db', 'changedb'];
Reconfigure.flags = Object.assign({}, base_1.default.flags, { 'no-strip-bom': command_1.flags.boolean({
        description: 'Do not remove the byte order mark from processed files',
    }), server: command_1.flags.string({
        char: 's',
        description: 'The database server name to change to. Defaults to `config.server`.',
    }), database: command_1.flags.string({
        char: 'd',
        description: 'The name of the database to change to. Defaults to `config.database`.',
    }), replace: command_1.flags.string({
        char: 'r',
        description: 'Replace entire connection string with this value instead of the current value.',
        exclusive: ['replace-raw'],
    }), 'replace-raw': command_1.flags.string({
        char: 'R',
        description: 'Same as -r but skips string replacements (the provided value is used verbatim).',
        exclusive: ['replace'],
    }) });
Reconfigure.examples = [
    chalk_1.default `$ lms reconfig   \n\t{gray # Replaces *server* and *database* values in connection strings with those from your current configuration \n\n\t This is useful for new databases with no auth info \n}`,
    chalk_1.default `$ lms reconf -r "Server=willbereplaced;DataBase=willbereplaced;UID=dev;PWD=elopment;"    \n\t{gray # Replaces entire connection string with this val (with *server* and *database* values replaced)}`,
    chalk_1.default `$ lms reconf -R "Server=willNOTbereplaced;DataBase=willNOTbereplaced;UID=dev;PWD=elopment;"    \n\t{gray # Replaces entire connection string with this val verbatim}`,
];
exports.default = Reconfigure;
